import os
import unittest
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch  # type: ignore

from varsnap import core, TestVarsnap


def add(x, y):
    return x + y


null = open(os.devnull, 'w')


class TestResult(unittest.runner.TextTestResult):
    def __init__(self, *args, **kwargs):
        super(TestResult, self).__init__(*args, **kwargs)
        self.successes = []

    def addSuccess(self, test):
        super(TestResult, self).addSuccess(test)
        self.successes.append(test)


class TestAssertGenerator(unittest.TestCase):
    def setUp(self):
        core.CONSUMERS = []

    def tearDown(self):
        core.CONSUMERS = []

    def run_test_case(self, test_case):
        suite = unittest.defaultTestLoader.loadTestsFromTestCase(test_case)
        runner = unittest.TextTestRunner(stream=null, resultclass=TestResult)
        return runner.run(suite)

    @patch('varsnap.core.Consumer.consume')
    def test_generate(self, mock_consume):
        core.Consumer(add)
        mock_consume.return_value = (True, '')
        result = self.run_test_case(TestVarsnap)
        self.assertEqual(len(result.errors), 0)
        self.assertEqual(len(result.failures), 0)
        self.assertEqual(len(result.skipped), 0)
        self.assertEqual(len(result.successes), 1)

    @patch('varsnap.core.Consumer.consume')
    def test_generate_failure(self, mock_consume):
        core.Consumer(add)
        mock_consume.return_value = (False, '')
        result = self.run_test_case(TestVarsnap)
        self.assertEqual(len(result.errors), 0)
        self.assertEqual(len(result.failures), 1)
        self.assertEqual(len(result.skipped), 0)
        self.assertEqual(len(result.successes), 0)

    @patch('varsnap.core.Consumer.consume')
    def test_generate_skipped(self, mock_consume):
        core.Consumer(add)
        mock_consume.return_value = None
        result = self.run_test_case(TestVarsnap)
        self.assertEqual(len(result.errors), 0)
        self.assertEqual(len(result.failures), 0)
        self.assertEqual(len(result.skipped), 1)
        self.assertEqual(len(result.successes), 0)



import logging
import sys
import unittest

from qualname import qualname

from . import core


varsnap_logger = logging.getLogger(core.__name__)
varsnap_logger.handlers = []
varsnap_logger.disabled = True
varsnap_logger.propagate = False

test_logger = logging.getLogger(__name__)
test_logger.setLevel(logging.INFO)
handler = logging.StreamHandler(sys.stdout)
test_logger.addHandler(handler)


class TestVarsnap(unittest.TestCase):
    def test_varsnap(self):
        results = []
        for consumer in core.CONSUMERS:
            consumer_name = qualname(consumer.target_func)
            test_logger.info("Running Varsnap tests for %s" % consumer_name)
            consumer.last_snap_id = None
            result = consumer.consume()
            if not result:
                continue
            results.append(result)
        if not results:
            raise unittest.case.SkipTest('No Snaps found')
        results = [x[1] for x in results if not x[0]]
        if not results:
            self.assertTrue(True)
            return
        result_log = "\n\n".join(results)
        self.assertTrue(False, result_log)



import base64
import binascii
try:
    import collections.abc as collections
except ImportError:  # Python 2 compatibility
    import collections  # type: ignore
import json
try:
    from json.decode import JSONDecodeError
except ImportError:  # Python 2 compatibility
    JSONDecodeError = ValueError
    ModuleNotFoundError = ImportError
import logging
import os
import pickle
import six
import sys
import threading
import time
import traceback
from typing import Any, Callable, List, Mapping, Optional, Tuple

from qualname import qualname
import requests

from .__version__ import __version__

PRODUCE_SNAP_URL = 'https://www.varsnap.com/api/snap/produce/'
CONSUME_SNAP_URL = 'https://www.varsnap.com/api/snap/consume/'
PRODUCE_TRIAL_URL = 'https://www.varsnap.com/api/trial/produce/'
UNPICKLE_ERRORS = [
    binascii.Error,
    ImportError,
    ModuleNotFoundError,
    pickle.UnpicklingError,
]
PICKLE_ERRORS = [
    AttributeError,
    pickle.PicklingError,
    TypeError,
]

# Names of different environment variables used by varsnap
# See readme for descriptions
ENV_VARSNAP = 'VARSNAP'
ENV_ENV = 'ENV'
ENV_PRODUCER_TOKEN = 'VARSNAP_PRODUCER_TOKEN'
ENV_CONSUMER_TOKEN = 'VARSNAP_CONSUMER_TOKEN'

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)
handler = logging.StreamHandler(sys.stdout)
LOGGER.addHandler(handler)

# A list of Varsnap functions for testing and tracing
CONSUMERS = []
PRODUCERS = []


def env_var(env):  # type: (str) -> str
    return os.environ.get(env, '').lower()


def get_signature(target_func):  # type: (Callable) -> str
    return 'python.%s.%s' % (__version__, qualname(target_func))


def equal(x, y):  # type: (Any, Any) -> bool
    if not isinstance(x, y.__class__):
        return False
    if isinstance(x, six.string_types):
        return x == y
    if isinstance(x, collections.Sequence):
        if len(x) != len(y):
            return False
        for v in zip(x, y):
            if not equal(v[0], v[1]):
                return False
        return True
    if isinstance(x, collections.Mapping):
        if len(x) != len(y):
            return False
        for k in x.keys():
            if k not in y:
                return False
            if not equal(x[k], y[k]):
                return False
        return True
    if hasattr(x, '__dict__'):
        return equal(x.__dict__, y.__dict__)
    return x == y


def align_report(report):  # type: (List[Tuple[str, str]]) -> str
    # Vertically align report's second column
    key_length = max([len(x[0]) for x in report]) + 2
    report_lines = [
        x[0] + ' '*(key_length - len(x[0])) + str(x[1])
        for x in report
    ]
    log = "\n".join(report_lines)
    return log


def limit_string(x):  # type: (str) -> str
    limit = 30
    ellipsis = '...'
    if len(x) <= limit:
        return x
    return x[:limit - len(ellipsis)] + ellipsis


class DeserializeError(ValueError):
    pass


class SerializeError(ValueError):
    pass


class Producer():
    def __init__(self, target_func):  # type: (Callable) -> None
        self.target_func = target_func
        PRODUCERS.append(self)

    @staticmethod
    def is_enabled():  # type: () -> bool
        if env_var(ENV_VARSNAP) != 'true':
            return False
        if env_var(ENV_ENV) != 'production':
            return False
        if not env_var(ENV_PRODUCER_TOKEN):
            return False
        return True

    @staticmethod
    def serialize(data):  # type: (Any) -> str
        try:
            pickle_data = pickle.dumps(data)
        except Exception as e:
            if type(e) in PICKLE_ERRORS:
                raise SerializeError(e)
            raise
        data = base64.b64encode(pickle_data).decode('utf-8')
        return data

    @staticmethod
    def get_globals():  # type: () -> Mapping[str, Any]
        global_vars = {}
        for k, v in globals().items():
            if k[:2] == '__':
                continue
            try:
                pickle.dumps(v)
            # Ignore unpickable data
            except Exception as e:
                if type(e) in PICKLE_ERRORS:
                    continue
                raise
            global_vars[k] = v
        return global_vars

    def produce(self, args, kwargs, output):  # type: (Any, Any, Any) -> None
        if not Producer.is_enabled():
            return
        LOGGER.info(
            'Varsnap producing call for %s' %
            qualname(self.target_func)
        )
        global_vars = Producer.get_globals()
        inputs = {
            'args': args,
            'kwargs': kwargs,
            'globals': global_vars
        }
        try:
            data = {
                'producer_token': env_var(ENV_PRODUCER_TOKEN),
                'signature': get_signature(self.target_func),
                'inputs': Producer.serialize(inputs),
                'prod_outputs': Producer.serialize(output)
            }
        except SerializeError:
            return
        requests.post(PRODUCE_SNAP_URL, data=data)


class Consumer():
    def __init__(self, target_func):  # type: (Callable) -> None
        self.target_func = target_func
        self.last_snap_id = None
        CONSUMERS.append(self)

    @staticmethod
    def is_enabled():  # type: () -> bool
        if env_var(ENV_VARSNAP) != 'true':
            return False
        if env_var(ENV_ENV) != 'development':
            return False
        if not env_var(ENV_CONSUMER_TOKEN):
            return False
        return True

    @staticmethod
    def deserialize(data):  # type: (str) -> Any
        try:
            data = pickle.loads(base64.b64decode(data.encode('utf-8')))
        except Exception as e:
            if type(e) in UNPICKLE_ERRORS:
                raise DeserializeError(e)
            raise
        return data

    def consume_watch(self):  # type: () -> None
        if not Consumer.is_enabled():
            return
        LOGGER.info(
            'Varsnap consuming calls to %s' %
            qualname(self.target_func)
        )
        while True:
            self.consume()
            time.sleep(1)

    def consume(self):  # type: () -> Tuple[bool, str]
        data = {
            'consumer_token': env_var(ENV_CONSUMER_TOKEN),
            'signature': get_signature(self.target_func),
        }
        response = requests.post(CONSUME_SNAP_URL, data=data)
        try:
            response_data = json.loads(response.content)
        except JSONDecodeError:
            response_data = ''
        if not response_data or response_data['status'] != 'ok':
            return True, ""
        trial_results = []
        for result in response_data['results']:
            trial_result = self.consume_one(result)
            trial_results.append(trial_result)
        all_matches = all([x for x in trial_results if x is not None])
        logs = "\n\n".join([x[1] for x in trial_results])
        return all_matches, logs

    def consume_one(self, snap_data):
        # type: (Mapping) -> Tuple[Optional[bool], str]
        if snap_data['id'] == self.last_snap_id:
            return None, ''

        self.last_snap_id = snap_data['id']
        LOGGER.info(
            'Receiving call from Varsnap uuid: ' + str(self.last_snap_id)
        )
        try:
            inputs = Consumer.deserialize(snap_data['inputs'])
            prod_outputs = Consumer.deserialize(snap_data['prod_outputs'])
        except DeserializeError:
            return None, ''
        if type(inputs) == list:
            # Backwards compatibility of 0.8.1 to 0.8.0
            inputs = {
                'args': inputs[0],
                'kwargs': inputs[1],
                'globals': inputs[2],
            }
        exception = ''
        for k, v in inputs['globals'].items():
            globals()[k] = v
        try:
            local_outputs = self.target_func(
                *inputs['args'],
                **inputs['kwargs']
            )
        except Exception as e:
            local_outputs = e
            exception = traceback.format_exc()
        if exception:
            matches = equal(prod_outputs, exception)
        else:
            matches = equal(prod_outputs, local_outputs)
        report_lines = []  # type: List[Tuple[str, str]]
        report_lines += self.report_central(
            env_var(ENV_CONSUMER_TOKEN),
            snap_data['id'],
            local_outputs,
            matches,
        )
        report_lines += self.report_log(
            inputs, str(prod_outputs), str(local_outputs), exception, matches
        )
        report = align_report(report_lines)
        if matches:
            LOGGER.info(report)
        else:
            LOGGER.error(report)

        return matches, report

    def report_central(self, consumer_token, snap_id, local_outputs, matches):
        # type: (str, str, Any, bool) -> List[Tuple[str, str]]
        data = {
            'consumer_token': consumer_token,
            'snap_id': snap_id,
            'test_outputs': Producer.serialize(local_outputs),
            'matches': matches,
        }
        response = requests.post(PRODUCE_TRIAL_URL, data=data)
        try:
            response_data = json.loads(response.content)
        except JSONDecodeError:
            response_data = ''
        if response_data['status'] != 'ok':
            return []
        trial_url = response_data.get('trial_url', '')
        report_line = ('Report URL:', trial_url)
        return [report_line]

    def report_log(
        self, inputs, prod_outputs, local_outputs, exception, matches
    ):  # type: (Any, str, str, str, bool) -> List[Tuple[str, str]]
        function_name = qualname(self.target_func)
        report = []
        report.append(('Function:', function_name))
        report.append(('Function input args:', limit_string(inputs['args'])))
        report.append((
            'Function input kwargs:', limit_string(inputs['kwargs'])
        ))
        report.append((
            'Production function outputs:', limit_string(prod_outputs)
        ))
        report.append(('Your function outputs:', limit_string(local_outputs)))
        if exception:
            report.append(('Local exception:', limit_string(exception)))
        report.append(('Matching outputs:', matches))
        return report


def varsnap(func):  # type: (Callable) -> Callable
    producer = Producer(func)
    Consumer(func)

    def magic(*args, **kwargs):
        try:
            output = func(*args, **kwargs)
        except Exception as e:
            threading.Thread(
                target=producer.produce,
                args=(args, kwargs, e),
            ).start()
            raise
        threading.Thread(
            target=producer.produce,
            args=(args, kwargs, output),
        ).start()
        return output
    LOGGER.info('Varsnap Loaded')
    # Reuse the original function name so it works with flask handlers
    magic.__name__ = func.__name__
    return magic



import json
import logging
import os
import sys
import time
import unittest
try:
    from unittest.mock import patch
    from unittest.mock import MagicMock
except ImportError:
    from mock import patch  # type: ignore
    from mock import MagicMock

from varsnap import core
from varsnap.__version__ import __version__


logger = logging.getLogger(core.__name__)
logger.disabled = True


class EnvVar(unittest.TestCase):
    def setUp(self):
        self.orig_varsnap = os.environ.get(core.ENV_VARSNAP, '')
        self.orig_env = os.environ.get(core.ENV_ENV, '')
        self.orig_producer_token = os.environ.get(core.ENV_PRODUCER_TOKEN, '')
        self.orig_consumer_token = os.environ.get(core.ENV_CONSUMER_TOKEN, '')
        core.CONSUMERS = []
        core.PRODUCERS = []

    def tearDown(self):
        os.environ[core.ENV_VARSNAP] = self.orig_varsnap
        os.environ[core.ENV_ENV] = self.orig_env
        os.environ[core.ENV_PRODUCER_TOKEN] = self.orig_producer_token
        os.environ[core.ENV_CONSUMER_TOKEN] = self.orig_consumer_token


class TestEnvVar(EnvVar):
    def test_env_var(self):
        os.environ[core.ENV_VARSNAP] = 'true'
        env = core.env_var(core.ENV_VARSNAP)
        self.assertEqual(env, 'true')

    def test_downcases_env_var(self):
        os.environ[core.ENV_VARSNAP] = 'TRUE'
        env = core.env_var(core.ENV_VARSNAP)
        self.assertEqual(env, 'true')

    def test_unset_var(self):
        del os.environ[core.ENV_VARSNAP]
        env = core.env_var(core.ENV_VARSNAP)
        self.assertEqual(env, '')


class TestEqual(EnvVar):
    def test_primitives(self):
        self.assertTrue(core.equal(1, 1))
        self.assertFalse(core.equal(1, '1'))
        self.assertTrue(core.equal('asdf', 'asdf'))

    def test_iterables(self):
        self.assertTrue(core.equal([1, 2, 3], [1, 2, 3]))
        self.assertTrue(core.equal({1: 'a', 2: 'b'}, {1: 'a', 2: 'b'}))
        self.assertFalse(core.equal([1, 2], [1, 2, 3]))
        self.assertFalse(core.equal({1: 2}, {1: 3}))
        self.assertFalse(core.equal({1: 2}, {2: 2}))

    def test_objects(self):
        class X():
            def __init__(self, x):
                self.x = x
        self.assertTrue(core.equal(X('asdf'), X('asdf')))
        self.assertFalse(core.equal(X('asdf'), X('qwer')))
        self.assertTrue(core.equal(X(X('asdf')), X(X('asdf'))))


class TestGetSignature(unittest.TestCase):
    def assertEqualVersion(self, signature, expected):
        self.assertEqual(signature, expected % __version__)

    def test_standalone_func(self):
        signature = core.get_signature(core.env_var)
        self.assertEqualVersion(signature, 'python.%s.env_var')

    def test_class_func(self):
        signature = core.get_signature(core.Producer.serialize)
        self.assertEqualVersion(signature, 'python.%s.Producer.serialize')

    def test_instance_func(self):
        signature = core.get_signature(core.Producer.__init__)
        self.assertEqualVersion(signature, 'python.%s.Producer.__init__')


class TestLimitString(unittest.TestCase):
    def test_small_string(self):
        x = 'asdf'
        self.assertEqual(core.limit_string(x), x)

    def test_medium_string(self):
        x = 'x' * 30
        self.assertEqual(core.limit_string(x), x)

    def test_long_string(self):
        x = 'x' * 50
        limited = core.limit_string(x)
        self.assertEqual(limited[:27], 'x' * 27)
        self.assertEqual(limited[27:], '...')


class TestProducer(EnvVar):
    def setUp(self):
        super(TestProducer, self).setUp()
        os.environ[core.ENV_VARSNAP] = 'true'
        os.environ[core.ENV_ENV] = 'production'
        os.environ[core.ENV_PRODUCER_TOKEN] = 'asdf'

        self.producer = core.Producer(core.env_var)

    def test_init(self):
        target_func = MagicMock()
        producer = core.Producer(target_func)
        self.assertEqual(producer.target_func, target_func)
        self.assertIn(producer, core.PRODUCERS)

    def test_is_enabled(self):
        self.assertTrue(core.Producer.is_enabled())

        os.environ[core.ENV_VARSNAP] = 'false'
        self.assertFalse(core.Producer.is_enabled())
        os.environ[core.ENV_VARSNAP] = 'true'

        os.environ[core.ENV_ENV] = 'development'
        self.assertFalse(core.Producer.is_enabled())
        os.environ[core.ENV_ENV] = 'production'

        os.environ[core.ENV_PRODUCER_TOKEN] = ''
        self.assertFalse(core.Producer.is_enabled())
        os.environ[core.ENV_PRODUCER_TOKEN] = 'asdf'

    def test_serialize(self):
        data = core.Producer.serialize('abcd')
        self.assertGreater(len(data), 0)

    @patch('requests.post')
    def test_produce_not_enabled(self, mock_post):
        os.environ[core.ENV_VARSNAP] = 'false'
        self.producer.produce('a', 'b', 'c')
        self.assertFalse(mock_post.called)

    @patch('requests.post')
    def test_produce(self, mock_post):
        self.producer.produce('a', 'b', 'c')
        self.assertEqual(mock_post.call_args[0][0], core.PRODUCE_SNAP_URL)
        data = mock_post.call_args[1]['data']
        self.assertEqual(data['producer_token'], 'asdf')
        self.assertEqual(data['signature'], core.get_signature(core.env_var))
        self.assertIn('inputs', data)
        self.assertIn('prod_outputs', data)


class TestConsumer(EnvVar):
    def setUp(self):
        super(TestConsumer, self).setUp()
        os.environ[core.ENV_VARSNAP] = 'true'
        os.environ[core.ENV_ENV] = 'development'
        os.environ[core.ENV_CONSUMER_TOKEN] = 'asdf'

        self.target_func = MagicMock()
        self.consumer = core.Consumer(self.target_func)

    def test_init(self):
        target_func = MagicMock()
        consumer = core.Consumer(target_func)
        self.assertEqual(consumer.target_func, target_func)
        self.assertIn(consumer, core.CONSUMERS)

    def test_is_enabled(self):
        self.assertTrue(core.Consumer.is_enabled())

        os.environ[core.ENV_VARSNAP] = 'false'
        self.assertFalse(core.Consumer.is_enabled())
        os.environ[core.ENV_VARSNAP] = 'true'

        os.environ[core.ENV_ENV] = 'production'
        self.assertFalse(core.Consumer.is_enabled())
        os.environ[core.ENV_ENV] = 'development'

        os.environ[core.ENV_CONSUMER_TOKEN] = ''
        self.assertFalse(core.Consumer.is_enabled())
        os.environ[core.ENV_CONSUMER_TOKEN] = 'asdf'

    def test_deserialize(self):
        data = core.Producer.serialize('abcd')
        output = core.Consumer.deserialize(data)
        self.assertEqual(output, 'abcd')

        data = core.Producer.serialize(EnvVar)
        output = core.Consumer.deserialize(data)
        self.assertEqual(output, EnvVar)

    def test_deserialize_known_error(self):
        with self.assertRaises(core.DeserializeError):
            core.Consumer.deserialize('abcd')

    @patch('pickle.loads')
    def test_deserialize_unknown_error(self, mock_loads):
        mock_loads.side_effect = MemoryError('asdf')
        with self.assertRaises(MemoryError):
            core.Consumer.deserialize('abcd')

    @patch('requests.post')
    def test_consume_not_enabled(self, mock_post):
        os.environ[core.ENV_VARSNAP] = 'false'
        self.consumer.consume_watch()
        self.assertFalse(mock_post.called)

    @patch('varsnap.core.qualname')
    @patch('requests.post')
    def test_consume_empty(self, mock_post, mock_qualname):
        mock_qualname.return_value = 'magicmock'
        mock_post.return_value = MagicMock(content='')
        self.consumer.consume()
        self.assertFalse(self.target_func.called)

    @patch('varsnap.core.qualname')
    @patch('requests.post')
    def test_consume(self, mock_post, mock_qualname):
        mock_qualname.return_value = 'magicmock'
        inputs = {
            'args': (2,),
            'kwargs': {},
            'globals': {},
        }
        data = {
            'results': [{
                'id': 'abcd',
                'inputs': core.Producer.serialize(inputs),
                'prod_outputs': core.Producer.serialize((4,)),
            }],
            'status': 'ok',
        }
        data = json.dumps(data)
        mock_post.return_value = MagicMock(content=data)
        self.target_func.return_value = (4,)
        self.consumer.consume()
        self.assertEqual(self.target_func.call_count, 1)
        self.assertEqual(self.target_func.call_args[0][0], 2)
        snap_consume_request = mock_post.mock_calls[0][2]['data']
        self.assertEqual(snap_consume_request['consumer_token'], 'asdf')
        signature = core.get_signature(core.env_var)
        self.assertEqual(snap_consume_request['signature'], signature)
        trial_produce_request = mock_post.mock_calls[1][2]['data']
        self.assertEqual(trial_produce_request['consumer_token'], 'asdf')
        self.assertEqual(trial_produce_request['snap_id'], 'abcd')
        test_outputs = core.Producer.serialize((4,))
        self.assertEqual(trial_produce_request['test_outputs'], test_outputs)
        self.assertEqual(trial_produce_request['matches'], True)

    @patch('varsnap.core.qualname')
    @patch('requests.post')
    def test_consume_deduplicates(self, mock_post, mock_qualname):
        mock_qualname.return_value = 'magicmock'
        inputs = {
            'args': (2,),
            'kwargs': {},
            'globals': {},
        }
        data = {
            'results': [{
                'id': 'abcd',
                'inputs': core.Producer.serialize(inputs),
                'prod_outputs': core.Producer.serialize((4,)),
            }],
            'status': 'ok',
        }
        data = json.dumps(data)
        mock_post.side_effect = [
            MagicMock(content=data),
            MagicMock(content=data),
        ]
        self.target_func.return_value = (4,)
        with self.assertRaises(StopIteration):
            self.consumer.consume_watch()
        self.assertEqual(self.target_func.call_count, 1)

    @patch('varsnap.core.qualname')
    @patch('requests.post')
    def test_consume_catches_exceptions(self, mock_post, mock_qualname):
        mock_qualname.return_value = 'magicmock'
        inputs = {
            'args': (2,),
            'kwargs': {},
            'globals': {},
        }
        data = {
            'results': [{
                'id': 'abcd',
                'inputs': core.Producer.serialize(inputs),
                'prod_outputs': core.Producer.serialize((4,)),
            }],
            'status': 'ok',
        }
        data = json.dumps(data)
        mock_post.side_effect = [
            MagicMock(content=data),
            MagicMock(content=json.dumps({'status': 'ok'})),
        ]
        self.target_func.side_effect = ValueError('asdf')
        self.consumer.consume()
        self.assertEqual(self.target_func.call_count, 1)


class TestVarsnap(EnvVar):
    @patch('requests.post')
    def test_no_op(self, mock_post):
        os.environ[core.ENV_VARSNAP] = 'false'
        mock_func = MagicMock()
        mock_func.__name__ = 'mock_func'
        mock_func = core.varsnap(mock_func)
        mock_func(1)
        self.assertFalse(mock_post.called)

    @patch('varsnap.core.Consumer.consume_watch')
    @patch('varsnap.core.Producer.produce')
    def test_consume(self, mock_produce, mock_consume):
        if sys.version_info.major < 3:
            # TODO remove this
            return
        mock_func = MagicMock()
        mock_func.__name__ = 'mock_func'
        mock_func.return_value = 2
        varsnap_mock_func = core.varsnap(mock_func)
        result = varsnap_mock_func(1)
        self.assertEqual(result, 2)
        self.assertEqual(mock_func.call_count, 1)
        self.assertEqual(mock_consume.call_count, 0)
        self.assertEqual(mock_produce.call_count, 1)
        self.assertEqual(mock_produce.call_args[0][2], 2)

    @patch('varsnap.core.Consumer.consume_watch')
    @patch('varsnap.core.Producer.produce')
    def test_consume_exception(self, mock_produce, mock_consume):
        if sys.version_info.major < 3:
            # TODO remove this
            return
        mock_func = MagicMock()
        mock_func.__name__ = 'mock_func'
        mock_func.side_effect = ValueError('asdf')
        varsnap_mock_func = core.varsnap(mock_func)
        with self.assertRaises(ValueError):
            varsnap_mock_func(1)
        time.sleep(0.1)  # Make sure mock_produce has enough time to be called
        self.assertEqual(mock_func.call_count, 1)
        self.assertEqual(mock_consume.call_count, 0)
        self.assertEqual(mock_produce.call_count, 1)
        self.assertEqual(str(mock_produce.call_args[0][2]), 'asdf')

    @patch('varsnap.core.Consumer.consume_watch')
    @patch('varsnap.core.Producer.produce')
    def test_func_name(self, mock_produce, mock_consume):
        varsnap_func = core.varsnap(core.Producer.serialize)
        self.assertEqual(varsnap_func.__name__, 'serialize')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from codecs import open
from os import path


# Get the long description from the README file
here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

about = {}
with open(path.join(here, 'varsnap', '__version__.py')) as f:
    exec(f.read(), about)

setup(
    name='varsnap',

    version=about['__version__'],

    description='Varsnap',
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://www.varsnap.com/',

    author='Varsnap',
    author_email='admin@varsnap.com',

    classifiers=[
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Topic :: Software Development',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    keywords='',

    packages=find_packages(exclude=["tests"]),

    install_requires=[
        'requests>=2.0,<3.0',
        'qualname>=0.1.0,<0.2.0',
    ],

    test_suite="varsnap.tests",

    # testing requires flake8 and coverage but they're listed separately
    # because they need to wrap setup.py
    extras_require={
        'dev': [],
        'test': [],
    },
)
